Imports System.ComponentModel

''' <summary>
''' Test child object for a collection.
''' </summary>
Public Class OfficeItem

  Implements IEditableObject

  ''' <summary>
  ''' Raised to indicate that a property value has changed.
  ''' </summary>
  ''' <remarks>
  ''' NOTE that this event is not used by Windows Forms data binding.
  ''' It is used here so the child objects can notify the collection
  ''' object that data has changed.
  ''' </remarks>
  Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)
  ''' <summary>
  ''' Raised to indicate that the Name property has changed.
  ''' </summary>
  ''' <remarks>
  ''' This is implemented for completeness in case this object were
  ''' ever directly bound to a Windows form.
  ''' </remarks>
  Public Event NameChanged As EventHandler
  ''' <summary>
  ''' Raised to indicate that the Type property has changed.
  ''' </summary>
  ''' <remarks>
  ''' This is implemented for completeness in case this object were
  ''' ever directly bound to a Windows form.
  ''' </remarks>
  Public Event TypeChanged As EventHandler

  Private mParent As OfficeItems
  Private mName As String = ""
  Private mType As String = ""

  ''' <summary>
  ''' Creates an object from existing data. The object is
  ''' not considered new.
  ''' </summary>
  ''' <param name="parent">The collection containing this object.</param>
  ''' <param name="name">The name of the item.</param>
  ''' <param name="type">The type of the item.</param>
  Public Sub New(ByVal parent As OfficeItems, ByVal name As String, ByVal type As String)
    mParent = parent
    mName = name
    mType = type
    mIsNew = False
  End Sub

  ''' <summary>
  ''' Creates a new instance. The object is considered to be
  ''' new.
  ''' </summary>
  ''' <param name="parent">The collection containing this object.</param>
  Public Sub New(ByVal parent As OfficeItems)
    mParent = parent
    mName = "<not set>"
    mType = "<unknown>"
    mIsNew = True
  End Sub

  ''' <summary>
  ''' Gets or sets the name of the item.
  ''' </summary>
  Public Property Name() As String
    Get
      Return mName
    End Get
    Set(ByVal Value As String)
      mName = Value
      RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Name"))
      RaiseEvent NameChanged(Me, EventArgs.Empty)
    End Set
  End Property

  ''' <summary>
  ''' Gets or sets the type of the item.
  ''' </summary>
  Public Property Type() As String
    Get
      Return mType
    End Get
    Set(ByVal Value As String)
      mType = Value
      RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Type"))
      RaiseEvent TypeChanged(Me, EventArgs.Empty)
    End Set
  End Property

#Region " IEditableObject "

  Private mEditing As Boolean
  Private mIsNew As Boolean = True
  Private mOldName As String
  Private mOldType As String

  Private Sub BeginEdit() Implements System.ComponentModel.IEditableObject.BeginEdit
    If Not mEditing Then
      mOldName = mName
      mOldType = mType
      mEditing = True
    End If
  End Sub

  Private Sub CancelEdit() Implements System.ComponentModel.IEditableObject.CancelEdit
    If mEditing Then
      If mIsNew Then mParent.RemoveChild(Me)
      mName = mOldName
      mType = mOldType
      mEditing = False
    End If
  End Sub

  Private Sub EndEdit() Implements System.ComponentModel.IEditableObject.EndEdit
    mEditing = False
    mIsNew = False
  End Sub

#End Region

End Class
